@@ -19,16 +19,16 @@ updateModel <- function(model, adj){
   }
   if(length(grep('ghype', callname))>0){
     callname <- 'ghype'
-    newcall <- call(name = callname, graph=adj, directed=model$directed, selfloops=model$selfloops, xi=xi, unbiased=all(model$omega==1), regular=model$regular)
+    newcall <- call(name = callname, graph=adj, directed=model$directed, selfloops=model$selfloops, xi=xi, unbiased=all(model$omega==1), regular=model$regular, multinomial = TRUE)
   } else{
     if(length(grep('bccm', callname))>0){
       callname <- 'bccm'
-      newcall <- call(name = callname, adj=adj, labels=model$labels, directed=model$directed, selfloops=model$selfloops, xi=xi, regular=model$regular, directedBlocks=model$directedBlocks, homophily=model$homophily,inBlockOnly=model$inBlockOnly)
+      newcall <- call(name = callname, adj=adj, labels=model$labels, directed=model$directed, selfloops=model$selfloops, xi=xi, regular=model$regular, directedBlocks=model$directedBlocks, homophily=model$homophily,inBlockOnly=model$inBlockOnly, multinomial = TRUE)
     }
     if(length(grep('nrm', callname))>0){
       callname <- 'nrm'
       newcall <- call(name = callname, w=model$predictors, adj=adj, directed=model$directed,
-                      selfloops=model$selfloops, xi=xi, init = c(model$coef[-length(model$coef)],0.01), ci=FALSE, regular=model$regular)
+                      selfloops=model$selfloops, xi=xi, init = c(model$coef[-length(model$coef)],0.01), ci=FALSE, regular=model$regular, multinomial = TRUE)
     }
   }
 
@@ -49,6 +49,9 @@ check_specs <- function(object, ...){
 
 check_specs.matrix <- function(object, ...){
   if(is.matrix(object)){
+    if(nrow(object) != ncol(object)){
+      return(c('directed'=TRUE, 'selfloops'=FALSE, 'bipartite' = TRUE))
+    }
     # if(is.null(directed)){
       if(isSymmetric(object)){
         directed <- FALSE
@@ -71,5 +74,5 @@ check_specs.matrix <- function(object, ...){
       }
     # }
   }
-  return(c('directed'=directed, 'selfloops'=selfloops))
+  return(c('directed'=directed, 'selfloops'=selfloops, 'bipartite' = FALSE))
 }
\ No newline at end of file
